import os
import json
# import csv
import matplotlib.pyplot as plt

def count_unique_urls_per_ip(folder_path):
    """
    Reads all JSON files in a folder, extracts unique URLs for each IP (file name),
    and returns a dictionary mapping IPs to the count of unique URLs they visited.
    """
    ip_url_count = {}

    # Loop through all files in the folder
    for file_name in os.listdir(folder_path):
        file_path = os.path.join(folder_path, file_name)
        
        if os.path.isfile(file_path) and file_name.count('.') >= 3:  # Basic IP format check
            unique_urls = set()
            
            try:
                with open(file_path, 'r', encoding='utf-8') as file:
                    for line in file:
                        try:
                            log_entry = json.loads(line)
                            url = log_entry.get("url_hostname")
                            if url:
                                unique_urls.add(url)
                        except json.JSONDecodeError:
                            continue
            
            except Exception as e:
                print(f"Error reading file {file_name}: {e}")
                continue
            
            ip_url_count[file_name] = len(unique_urls)  # Store count of unique URLs

    return dict(sorted(ip_url_count.items(), key=lambda item: item[1], reverse=True))  # Sort in decreasing order

# def save_csv(ip_url_count, output_csv_path):
    # """
    # Saves the sorted IP-to-unique-URL count mapping into a CSV file.
    # """
    #with open(output_csv_path, 'w', newline='', encoding='utf-8') as csv_file:
     #   csv_writer = csv.writer(csv_file)
    #    csv_writer.writerow(["IP Address", "Unique URL Count"])
   #     
  #      for ip, count in ip_url_count.items():
 #           csv_writer.writerow([ip, count])
    
    # print(f"Results saved to {output_csv_path}") 

def generate_chart(ip_url_count, output_chart_path):
    """
    Generates a sorted bar chart with IPs on X-axis (hidden) and unique URL counts on Y-axis.
    """
    if not ip_url_count:
        print("No data available for chart.")
        return

    ips = list(ip_url_count.keys())
    url_counts = list(ip_url_count.values())

    # Create the bar chart
    plt.figure(figsize=(14, 6))
    plt.bar(range(len(ips)), url_counts, color='blue', edgecolor='black')
    plt.xlabel('IP Addresses (Hidden)')
    plt.ylabel('Unique URLs Visited')
    plt.title('Number of Unique URLs Visited by Each IP')
    
    # Hide X-axis labels for better visualization
    plt.xticks([])  

    plt.tight_layout()

    # Save the chart
    plt.savefig(output_chart_path)
    print(f"Chart saved to {output_chart_path}")
    plt.show()

if __name__ == "__main__":
    # Define input and output paths
    folder_path = r"C:\Users\aydan\Downloads\export\EXPORT"
    # output_csv_path = r"C:\Users\aydan\Desktop\MT\MT2\ip_url_counts.csv"
    output_chart_path = r"C:\Users\aydan\Desktop\MT\MT2\ip_url_chart.png"

    # Process JSON files
    ip_url_count = count_unique_urls_per_ip(folder_path)

    # Save to CSV
    # save_csv(ip_url_count, output_csv_path)

    # Generate bar chart
    generate_chart(ip_url_count, output_chart_path)
